#include <sparrowrenderer.h>
#include "mapscene.h"

#define HEX_TO_VEC3(hex) glm::vec3(float((hex & 0xFF0000) >> 16)/255, float((hex & 0x00FF00) >> 8)/255, float(hex & 0x0000FF)/255)

inline glm::vec3 getColor(Pixel::Type type)
{
    switch(type){
        case Pixel::BEDROCK : return HEX_TO_VEC3(0x101020);
        case Pixel::GRASS : return HEX_TO_VEC3(0x719678);
        case Pixel::MARK : return HEX_TO_VEC3(0x5D7B62);
        case Pixel::ROCK : return HEX_TO_VEC3(0x8C8C8C);
        case Pixel::IRON_ORE : return HEX_TO_VEC3(0x917B61);
        case Pixel::TREE : return HEX_TO_VEC3(0x003800);
        case Pixel::BERRIES : return HEX_TO_VEC3(0x4D6394);
        case Pixel::FOOD : return HEX_TO_VEC3(0xFF7A7A);
        case Pixel::WOOD : return HEX_TO_VEC3(0x634A22);
        case Pixel::STONE : return HEX_TO_VEC3(0x454545);
        case Pixel::IRON : return HEX_TO_VEC3(0x4A4036);
        case Pixel::DUDE :
                // TODO
                return HEX_TO_VEC3(0x0000FF);
        case Pixel::SPAWN :
                // TODO
                return HEX_TO_VEC3(0x0000FF);
        case Pixel::WALL : return HEX_TO_VEC3(0xE6B2A1);
        case Pixel::ROAD : return HEX_TO_VEC3(0xEDB287);
        case Pixel::SWORD : return HEX_TO_VEC3(0xEBEBEB);
        case Pixel::LIBRARY : return HEX_TO_VEC3(0xA37A50);
        case Pixel::DEAD_DUDE : return HEX_TO_VEC3(0xFF0000);
        default : return HEX_TO_VEC3(0x0000FF); // bleu absolu = bug
    }
}

MapScene::~MapScene()
{
    if(m_vao != 0)
    {
        glDeleteBuffers(1, &m_vbo);
        glDeleteVertexArrays(1, &m_vao);
        m_vao = 0;
    }
}

void MapScene::changePixel(const Coord &c, glm::vec3 color)
{
    m_pixels.push_back(Pix(glm::vec2(c.x, c.y), color));
}

void MapScene::changePixel(const Coord &c, Pixel::Type type)
{
    m_pixels.push_back(Pix(glm::vec2(c.x, c.y), getColor(type)));
}

void MapScene::initDraw()
{
    glGenVertexArrays(1, &m_vao);
    glBindVertexArray(m_vao);

    glGenBuffers(1, &m_vbo);
    glBindBuffer(GL_ARRAY_BUFFER, m_vbo);
    glEnableVertexAttribArray(0);
    glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, sizeof(Pix), (void*)(0));
    glEnableVertexAttribArray(1);
    glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, sizeof(Pix), (void*)(sizeof(glm::vec2)));
    glBindBuffer(GL_ARRAY_BUFFER, 0);

    glBindVertexArray(0);

    for(int i=0; i<getWidth(); ++i)
        for(int j=0; j<getHeight(); ++j)
            m_pixels.push_back(Pix(glm::vec2(i, j), getColor(getPixel(i, j).type)));
}

void MapScene::draw()
{
    glDisable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glBindVertexArray(m_vao);
    glBindBuffer(GL_ARRAY_BUFFER, m_vbo);
    glBufferData(GL_ARRAY_BUFFER, m_pixels.size() * sizeof(Pix), m_pixels.data(), GL_DYNAMIC_DRAW);
    glDrawArrays(GL_POINTS, 0, m_pixels.size());
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindVertexArray(0);
    m_pixels.clear();
}
